/*
 * Copyright (C) 1997-2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */
#include "para.h"
#include <pwd.h>
/*
 * do the usual stuff to become a daemon
 */
void daemon_init(void)
{
	pid_t pid;

	clog(INFO, "%s: daemonizing\n", __func__);
	if ((pid = fork()) < 0) {
		clog(EMERG, "%s: failed to init daemon\n", __func__);
		exit(EXIT_FAILURE);
	} else if (pid)
		exit(EXIT_SUCCESS); /* parent exits */
	/* child */
	setsid(); /* become session leader */
	chdir("/");
	umask(0);
}

/*
 * call exit() on errors this might be called from clog, so we must not
 * use clog...
 */
FILE *open_log(char *logfile_name)
{
	FILE *logfile;

	if (!logfile_name)
		return NULL;
	if (!(logfile = fopen(logfile_name, "a")))
		exit(EXIT_FAILURE);
	setlinebuf(logfile);
	return logfile;
}

/* logfile == NULL is ok */
void close_log(FILE* logfile)
{
	if (!logfile)
		return;
	clog(INFO, "%s: closing logfile\n", __func__);
	fclose(logfile);
}

void log_welcome(const char *whoami, int loglevel)
{
	/* use EMERG to get the separator always logged */
	clog(EMERG, "%s: \n", "");
	clog(INFO, "%s: welcome to %s " VERSION " ("BUILD_DATE") :\n",
		"", whoami);
	clog(INFO, "%s: \n", "");
	clog(DEBUG, "%s: using loglevel %d\n", __func__, loglevel);
}
void para_drop_privileges(const char *username)
{
	struct passwd *p;
	if (geteuid())
		return;
	if (!username) {
		clog(EMERG, "%s: root privileges, but no user option given\n", __func__);
		exit(EXIT_FAILURE);
	}
	p = getpwnam(username);
	if (!p) {
		clog(EMERG, "%s: no such user\n", __func__);
		exit(EXIT_FAILURE);
	}
	clog(NOTICE, "%s: dropping root privileges\n", __func__);
	setuid(p->pw_uid);
	clog(DEBUG, "%s: uid: %d, euid: %d\n", __func__, getuid(), geteuid());
	setuid(p->pw_uid);
}

/*
 * set or get server's uptime
 */
int server_uptime(int set_or_get)
{
	static time_t startuptime;
	time_t now;

	if (set_or_get == SET) {
		time(&startuptime);
		return 0;
	}
	time(&now);
	return((int) difftime(now, startuptime));
}
