#include "para.h"

extern const char *status_item_list[NUM_STAT_ITEMS];

/* FIXME: Fails badly if buf does not contain \n or \0 */

/* if line handler is NULL, return number of complete lines in buf.
 * Otherwise, call line_handler for each complete line, but no more than
 * num times. Copy rest of buffer (last chunk containing incomplete line)
 * to start of buffer and return its size.
 */
int check_buf_for_items(char *buf, int n, void (*line_handler)(char *), int num)
{
	char *start = buf, *end;
	int i, num_lines = 0;
	char tmp[STRINGSIZE];
	
	while (start <= buf + n) {
		char *next_null;
		char *next_cr;

		next_cr = memchr(start, '\n', buf + n - start);
		next_null = memchr(start, '\0', buf + n - start);
		if (next_null == start) { /* !?! */
			start++;
			continue;
		}
		if (!next_cr && !next_null)
			break;
		if (next_cr && next_null) {
			end = next_cr < next_null? next_cr : next_null;
		} else if (next_null) {
			end = next_null;
		} else
			end = next_cr;
		num_lines++;
		if (line_handler) {
			*end = '\0';
			line_handler(start);
			start = ++end;
			if (num && num_lines >= num)
				break;
		} else
			start = ++end;
	}
	if (!line_handler)
		return num_lines;
	i = buf + n - start;
	memcpy(tmp, start, i);
	memcpy(buf, tmp, i);
	return i;
}

/*
 * check if line starts with known command.
 */
int stat_line_valid(const char *line)
{
	int i;
	size_t line_len;

	if (!line || !*line)
		return -1;
	line_len = strlen(line);
	for (i = 0; i < NUM_STAT_ITEMS; i++) {
		const char *s = status_item_list[i];
		size_t item_len = strlen(s);
		
		if (line_len > item_len && line[item_len] == ':' &&
 				!strncmp(line, s, item_len))
			return i;
	}
	return -1;
}

int para_open_audiod_pipe(char *cmd)
{
	int fds[3] = {0, 1, 0};
	pid_t pid;
	return para_exec_cmdline_pid(&pid, cmd, fds) > 0?
		fds[1] : -1;
}

int read_audiod_pipe(int fd, void (*line_handler)(char *) )
{
	static char buf[STRINGSIZE + 1];
	static ssize_t loaded, bufsize = sizeof(buf);
	ssize_t ret;

	if (loaded >= bufsize)
		loaded = 0;
	ret = read(fd, buf + loaded, bufsize - loaded);
	if (ret > 0) {
		loaded += ret;
		buf[loaded] = '\0';
		loaded = check_buf_for_items(buf, loaded, line_handler, 0);
	}
	return ret;
}
